(function inject() {
  let prefs = {
    disableAutoplay: true,
    disablePreload: true,
  };

  let started = false;

  const originalMediaPrototypePlay = HTMLMediaElement.prototype.play;

  const userInputMonitor = {
    keydown: false,
    mousedown: false,
    touchstart: false,
    events: [['keydown', 'keyup'], ['mousedown', 'mouseup'], ['touchstart', 'touchend']],
    inputStatus: { keydown: false, mousedown: false, touchstart: false },
    listeners: {},

    init() {
      this.events.forEach((i) => {
        this.listeners[i[0]] = () => { this.inputStatus[i[0]] = true; };
        this.listeners[i[1]] = () => setTimeout(() => { this.inputStatus[i[0]] = false; }, 1000);
        window.addEventListener(i[0], this.listeners[i[0]], true);
        window.addEventListener(i[1], this.listeners[i[1]], true);
      });
    },

    uninit() {
      this.events.forEach((i) => {
        window.removeEventListener(i[0], this.listeners[i[0]], true);
        window.removeEventListener(i[1], this.listeners[i[1]], true);
      });
    },

    inputRecieved() {
      return this.inputStatus.keydown || this.inputStatus.mousedown || this.inputStatus.touchstart;
    },
  };

  function isJWPlayer(el) {
    return el.classList.contains('jw-video')
      && el.parentElement.classList.contains('jw-media')
      && el.parentElement.parentElement.classList.contains('jwplayer')
      && Object.prototype.hasOwnProperty.call(window, 'jwplayer');
  }

  function disableAutoplay(el) {
    if (isJWPlayer(el)) {
      const jwinstance = window.jwplayer(el.parentElement.parentElement);
      jwinstance.stop();
      return;
    }

    if (!prefs.disableAutoplay || !prefs.disablePreload || (!el.autoplay && el.preload === 'none')) {
      return;
    }

    if (el.autoplay) {
      el.autoplay = false;
    }

    if (el.preload !== 'none') {
      el.preload = 'none';
    }

    el.src = el.currentSrc;
  }

  function mediaPrototypePlay() {
    if (!prefs.disableAutoplay || userInputMonitor.inputRecieved()) {
      return originalMediaPrototypePlay.call(this);
    } else if (isJWPlayer(this)) {
      const jwinstance = window.jwplayer(this.parentElement.parentElement);
      jwinstance.pause();
    }
    return Promise.resolve();
  }

  function mediaLoadStart(event) {
    if (event.target instanceof HTMLMediaElement && !userInputMonitor.inputRecieved()) {
      disableAutoplay(event.target);
    }
  }

  function init(data) {
    prefs = data;
    document.querySelectorAll('audio, video').forEach(disableAutoplay);
    Object.defineProperty(HTMLMediaElement.prototype, 'play', {
      configurable: true,
      writable: false,
      enumerable: true,
      value: mediaPrototypePlay,
    });
  }

  function start() {
    userInputMonitor.init();
    window.addEventListener('loadstart', mediaLoadStart, true);
    started = true;
  }

  function stop() {
    userInputMonitor.uninit();
    window.removeEventListener('loadstart', mediaLoadStart, true);
    started = false;
  }

  window.addEventListener('message', (event) => {
    if (event.source !== window) {
      return;
    }

    switch (event.data.msg) {
      case 'dh5a:init': init(event.data.prefs); break;
      case 'dh5a:domain-prefs-changed': prefs[event.data.prefName] = event.data.value; break;
      default:
    }

    if (started && !prefs.disableAutoplay) {
      stop();
    } else if (!started && prefs.disableAutoplay) {
      start();
    }
  });

  postMessage({ msg: 'dh5a:script-injected' }, '*');
}());
